@@ -3,13 +3,15 @@
 # George Bouras, george.mpouras@yandex.com and Joan Ntzougani
 
 package Search::ElasticDirect;
+our $VERSION = '2.5.0';
+
+use strict;
+use warnings;
 use Search::ElasticDirect;
 use parent qw(Search::ElasticDirectHttp);
 use URI::Escape;
 use Cpanel::JSON::XS;
 
-our $VERSION = '2.2.0';
-
 my $json = Cpanel::JSON::XS->new;
 $json->allow_bignum(1);
 $json->allow_blessed(1);
@@ -26,11 +28,13 @@ my $response = '';
 my $options  = {};
 my $url      = '';
 
+
     #############
     #  Methods  #
     #############
 
-#   Create new object
+
+#   Create new object and provide a connection to the ElasticSearch using varius methods
 sub new
 {
 my $class = shift || __PACKAGE__;
@@ -39,7 +43,7 @@ my $self  = {
   host          => '127.0.0.1',
   port          => 9200,
   protocol      => 'https',
-  timeout       => 120,
+  timeout       => 360,
   scroll        => '5m',
   keep_alive    => 1,
   verify_SSL    => 0,
@@ -62,37 +66,117 @@ my $self  = {
   format        => 'data' # json, json_pretty, yaml, txt, data
 	};
 
-  # Set properties from arguments
+# Set properties from arguments
+
   for (my($i,$j)=(0,1); $i < scalar(@_) - (scalar(@_) % 2); $i+=2,$j+=2) {
 
     if ( exists $self->{$_[$i]} ) {
 
       if ('_' eq substr $_[$i],0,1) {
-      CORE::warn "Private property \"$_[$i]\" is read only. Valid user properties are : ", join(', ', sort grep ! /^_/, keys %{$self}) ,"\n";
-      return undef
+      $self->{error} = "Private property \"$_[$i]\" is read only. Valid user properties are : ". join(', ', sort grep ! /^_/, keys %{$self});
+      return $self
       }
 
     $self->{$_[$i]}=$_[$j]
     }
     else {
-    CORE::warn "Invalid property \"$_[$i]\" valid proerties are : ", join(', ', sort grep ! /^_/, keys %{$self}) ,"\n";
-    return undef
+    $self->{error} = "Invalid property \"$_[$i]\" valid proerties are : ". join(', ', sort grep ! /^_/, keys %{$self});
+    return $self
     }
   }
 
 $self->{authentication} = $self->{authentication} =~/(?i)t|y|1/ ? 1:0;
 $self->{keep_alive}     = $self->{keep_alive}     =~/(?i)t|y|1/ ? 1:0;
 $self->{verify_SSL}     = $self->{verify_SSL}     =~/(?i)t|y|1/ ? 1:0;
-$self->{PIT}        = $self->{PIT}        =~/(?i)t|y|1/ ? 1:0;
+$self->{PIT}            = $self->{PIT}            =~/(?i)t|y|1/ ? 1:0;
+$self->{protocol}     //= 'https';
 
-if ($self->{timeout} !~/^\d+$/) {CORE::warn "port ($self->{timeout}) should be an integer\n";      return undef}
-if ($self->{port}    !~/^\d+$/) {CORE::warn "port ($self->{port}) should be an integer\n";         return undef}
-if ($self->{port}    > 65535)   {CORE::warn "port ($self->{port}) should be an less than 65536\n"; return undef}
+if ($self->{timeout} !~/^\d+$/) {$self->{error} = "port ($self->{timeout}) should be an integer\n";      return $self}
+if ($self->{port}    !~/^\d+$/) {$self->{error} = "port ($self->{port}) should be an integer\n";         return $self}
+if ($self->{port}    > 65535)   {$self->{error} = "port ($self->{port}) should be an less than 65536\n"; return $self}
+
+  if ($self->{authentication}) {
+  $self->{username} = URI::Escape::uri_escape( $self->{username} );
+  $self->{password} = URI::Escape::uri_escape( $self->{password} );
+  }
+  else {
+  delete $self->{username};
+  delete $self->{password}
+  }
+
+
+  if ('https' eq $self->{protocol}) {
+
+    # https searchguard
+    if ( (! $self->{authentication}) && (defined $self->{certca}) && (defined $self->{cert}) && (defined $self->{key}) ) {
+    unless (-f $self->{certca}) {$self->{error} = "missing ssl certca file $self->{certca}"; return $self}
+    unless (-f $self->{cert}  ) {$self->{error} = "missing ssl cart file $self->{cert}";     return $self}
+    unless (-f $self->{key}   ) {$self->{error} = "missing ssl key file $self->{key}";       return $self}
+    $self->{connect}     = Search::ElasticDirectHttp::HTTP_Tiny_https_certca_cert_key($self->{timeout}, $self->{keep_alive}, $self->{verify_SSL}, $self->{certca}, $self->{cert}, $self->{key});
+    $self->{url}         = "https://$self->{host}:$self->{port}";
+    $self->{description} = 'SearchGuard, CA certificate, cert admin, key admin'
+    }
+
+    # https username, password, certificate. Usually you want this one
+    elsif ( ($self->{authentication}) && (defined $self->{certca}) && (defined $self->{username}) && (defined $self->{password}) ) {
+    unless (-f $self->{certca}) {$self->{error} = "missing ssl certca file $self->{certca}"; return $self}
+    $self->{connect}     = Search::ElasticDirectHttp::HTTP_Tiny_https_certca($self->{timeout}, $self->{keep_alive}, $self->{verify_SSL}, $self->{certca});
+    $self->{url}         = "https://$self->{username}:$self->{password}\@$self->{host}:$self->{port}";
+    $self->{description} = 'https, CA certificate, username, password';
+    $self->{cert}        = undef;
+    $self->{key}         = undef
+    }
+
+    # https username, password, when you have a signed host certificate
+    elsif ( ($self->{authentication}) && (defined $self->{username}) && (defined $self->{password}) ) {
+    $self->{connect}     = Search::ElasticDirectHttp::HTTP_Tiny_https($self->{timeout}, $self->{keep_alive}, $self->{verify_SSL});
+    $self->{url}         = "https://$self->{username}:$self->{password}\@$self->{host}:$self->{port}";
+    $self->{description} = 'https, username, password';
+    $self->{certca}      = undef;
+    $self->{cert}        = undef;
+    $self->{key}         = undef
+    }
+
+    else {
+    $self->{error} = "uknown $self->{protocol} connection mode";
+    return $self
+    }
+  }
+  elsif ('http' eq $self->{protocol}) {
+
+    # http username, password
+    if ( ($self->{authentication}) && (defined $self->{username}) && (defined $self->{password}) ) {
+    $self->{connect}     = Search::ElasticDirectHttp::HTTP_Tiny_http($self->{timeout}, $self->{keep_alive});
+    $self->{url}         = "http://$self->{username}:$self->{password}\@$self->{host}:$self->{port}";
+    $self->{description} = 'http, username, password'
+    }
+
+    # http
+    elsif (! $self->{authentication}) {
+    $self->{connect}     = Search::ElasticDirectHttp::HTTP_Tiny_http($self->{timeout}, $self->{keep_alive});
+    $self->{url}         = "http://$self->{host}:$self->{port}";
+    $self->{description} = 'http';
+    }
+
+    else {
+    $self->{error} = "unknown $self->{protocol} connection mode";
+    return $self
+    }
+  }
+
+  else {
+  $self->{error} = "protocol ( $self->{protocol} ) should be http or https";
+  return $self
+  }
+
+delete $self->{protocol};
 bless $self,$class
 }
 
+
 sub DESTROY {}
 
+
 #   Auto run when a non existing subroutine is called
 sub AUTOLOAD
 {
@@ -145,7 +229,12 @@ sub ActiveMaster
 {
 my $obj   = ((exists $_[0]) && (__PACKAGE__ eq ref $_[0])) ? shift : __NOT_AN_OBJECT([caller 0]->[3],@_);
 $response = $obj->{connect}->request('GET', "$obj->{url}/_cat/nodes?format=json&pretty=false&h=name,master", {headers=> {'Content-Type'=>'application/json'}});
-unless ($response->{success}) {CORE::die 'method : '. [caller 0]->[3]." , status : $response->{status} , error : $response->{reason} , content : $response->{content}\n"; return undef}
+$obj->{error}=0;
+
+  unless ($response->{success}) {
+  $obj->{error} = 'method : '. [caller 0]->[3]." , status : $response->{status} , error : $response->{reason} , content : $response->{content}\n";
+  return undef
+  }
 
   foreach (grep $_->{master} eq '*', @{ $json->decode($response->{content}) }) {
   return 1 if $_->{name} =~/^$obj->{__this_server}/
@@ -154,128 +243,13 @@ unless ($response->{success}) {CORE::die 'method : '. [caller 0]->[3]." , status
 }
 
 
-#   Provides a connection to the ElasticSearch using varius methods
-sub connect
-{
-my $obj       = ((exists $_[0]) && (__PACKAGE__ eq ref $_[0])) ? shift : __NOT_AN_OBJECT([caller 0]->[3],@_);
-my $HTTP_Tiny = $obj->clone;
-
-  # Set properties from arguments
-  for (my($i,$j)=(0,1); $i < scalar(@_) - (scalar(@_) % 2); $i+=2,$j+=2) {
-
-    if (exists $HTTP_Tiny->{$_[$i]}) {
-		
-      if ('_' eq substr $_[$i],0,1) {
-      $HTTP_Tiny->{error} = "Private property \"$_[$i]\" is read only. Valid user properties are : ". join(', ', sort grep ! /^(error|_)/, keys %{$HTTP_Tiny});
-      return undef
-      }
-
-    $HTTP_Tiny->{$_[$i]}=$_[$j]
-    }
-    else {
-    $HTTP_Tiny->{error} = "Invalid property $_[$i] valid proerties are : ". join(', ', sort grep ! /^(error|_)/, keys %{$HTTP_Tiny});
-    return undef
-    }
-  }
-
-$HTTP_Tiny->{protocol}    //= 'https';
-$HTTP_Tiny->{port}        //= 9200;
-$HTTP_Tiny->{error}         = 0;
-$HTTP_Tiny->{keep_alive}    = $HTTP_Tiny->{keep_alive}     =~/(?i)t|y|1/ ? 1:0;
-$HTTP_Tiny->{verify_SSL}    = $HTTP_Tiny->{verify_SSL}     =~/(?i)t|y|1/ ? 1:0;
-$HTTP_Tiny->{authentication}= $HTTP_Tiny->{authentication} =~/(?i)t|y|1/ ? 1:0;
-
-if ($HTTP_Tiny->{timeout} !~/^\d+$/) {$HTTP_Tiny->{error} = "port ($HTTP_Tiny->{timeout}) should be an integer";      return undef}
-if ($HTTP_Tiny->{port}    !~/^\d+$/) {$HTTP_Tiny->{error} = "port ($HTTP_Tiny->{port}) should be an integer";         return undef}
-if ($HTTP_Tiny->{port}    > 65535)   {$HTTP_Tiny->{error} = "port ($HTTP_Tiny->{port}) should be an less than 65536"; return undef}
-
-  if ($HTTP_Tiny->{authentication}) {
-  $HTTP_Tiny->{username} = URI::Escape::uri_escape( $HTTP_Tiny->{username} );
-  $HTTP_Tiny->{password} = URI::Escape::uri_escape( $HTTP_Tiny->{password} );
-  }
-  else {
-  $HTTP_Tiny->{username} = undef;
-  $HTTP_Tiny->{password} = undef;
-  }
-
-  if ($HTTP_Tiny->{protocol} eq 'https') {
-
-    # https searchguard
-    if ( (! $HTTP_Tiny->{authentication}) && (defined $HTTP_Tiny->{certca}) && (defined $HTTP_Tiny->{cert}) && (defined $HTTP_Tiny->{key}) ) {
-    unless (-f $HTTP_Tiny->{certca}) {$HTTP_Tiny->{error} = "missing ssl certca file $HTTP_Tiny->{certca}"; return undef}
-    unless (-f $HTTP_Tiny->{cert}  ) {$HTTP_Tiny->{error} = "missing ssl cart file $HTTP_Tiny->{cert}";     return undef}
-    unless (-f $HTTP_Tiny->{key}   ) {$HTTP_Tiny->{error} = "missing ssl key file $HTTP_Tiny->{key}";       return undef}
-    $HTTP_Tiny->{connect}     = Search::ElasticDirectHttp::HTTP_Tiny_https_certca_cert_key($HTTP_Tiny->{timeout}, $HTTP_Tiny->{keep_alive}, $HTTP_Tiny->{verify_SSL}, $HTTP_Tiny->{certca}, $HTTP_Tiny->{cert}, $HTTP_Tiny->{key});
-    $HTTP_Tiny->{url}         = "$HTTP_Tiny->{protocol}://$HTTP_Tiny->{host}:$HTTP_Tiny->{port}";
-    $HTTP_Tiny->{description} = 'SearchGuard, CA certificate, cert admin, key admin';
-    $HTTP_Tiny->{username}    = undef;
-    $HTTP_Tiny->{password}    = undef
-    }
-
-    # https username, password, certificate. Usually you want this one
-    elsif ( ($HTTP_Tiny->{authentication}) && (defined $HTTP_Tiny->{certca}) && (defined $HTTP_Tiny->{username}) && (defined $HTTP_Tiny->{password}) ) {
-    unless (-f $HTTP_Tiny->{certca}) {$HTTP_Tiny->{error} = "missing ssl certca file $HTTP_Tiny->{certca}"; return undef}
-    $HTTP_Tiny->{connect}     = Search::ElasticDirectHttp::HTTP_Tiny_https_certca($HTTP_Tiny->{timeout}, $HTTP_Tiny->{keep_alive}, $HTTP_Tiny->{verify_SSL}, $HTTP_Tiny->{certca});
-    $HTTP_Tiny->{url}         = "$HTTP_Tiny->{protocol}://$HTTP_Tiny->{username}:$HTTP_Tiny->{password}\@$HTTP_Tiny->{host}:$HTTP_Tiny->{port}";
-    $HTTP_Tiny->{description} = 'https, CA certificate, username, password';
-    $HTTP_Tiny->{cert}        = undef;
-    $HTTP_Tiny->{key}         = undef
-    }
-
-    # https username, password, when you have a signed host certificate
-    elsif ( ($HTTP_Tiny->{authentication}) && (defined $HTTP_Tiny->{username}) && (defined $HTTP_Tiny->{password}) ) {
-    $HTTP_Tiny->{connect}     = Search::ElasticDirectHttp::HTTP_Tiny_https($HTTP_Tiny->{timeout}, $HTTP_Tiny->{keep_alive}, $HTTP_Tiny->{verify_SSL});
-    $HTTP_Tiny->{url}         = "$HTTP_Tiny->{protocol}://$HTTP_Tiny->{username}:$HTTP_Tiny->{password}\@$HTTP_Tiny->{host}:$HTTP_Tiny->{port}";
-    $HTTP_Tiny->{description} = 'https, username, password';
-    $HTTP_Tiny->{certca}      = undef;
-    $HTTP_Tiny->{cert}        = undef;
-    $HTTP_Tiny->{key}         = undef
-    }
-
-    else {
-    $HTTP_Tiny->{error} = "uknown $HTTP_Tiny->{protocol} connection mode";
-    return undef
-    }
-  }
-  elsif ($HTTP_Tiny->{protocol} eq 'http') {
-
-    # http username, password
-    if ( ($HTTP_Tiny->{authentication}) && (defined $HTTP_Tiny->{username}) && (defined $HTTP_Tiny->{password}) ) {
-    $HTTP_Tiny->{connect}     = Search::ElasticDirectHttp::HTTP_Tiny_http($HTTP_Tiny->{timeout}, $HTTP_Tiny->{keep_alive});
-    $HTTP_Tiny->{url}         = "$HTTP_Tiny->{protocol}://$HTTP_Tiny->{username}:$HTTP_Tiny->{password}\@$HTTP_Tiny->{host}:$HTTP_Tiny->{port}";
-    $HTTP_Tiny->{description} = 'http, username, password';
-    }
-
-    # http
-    elsif (! $HTTP_Tiny->{authentication}) {
-    $HTTP_Tiny->{connect}     = Search::ElasticDirectHttp::HTTP_Tiny_http($HTTP_Tiny->{timeout}, $HTTP_Tiny->{keep_alive});
-    $HTTP_Tiny->{url}         = "$HTTP_Tiny->{protocol}://$HTTP_Tiny->{host}:$HTTP_Tiny->{port}";
-    $HTTP_Tiny->{description} = 'http';
-    }
-
-    else {
-    $HTTP_Tiny->{error} = "uknown $HTTP_Tiny->{protocol} connection mode";
-    return undef
-    }
-  }
-
-  else {
-  $HTTP_Tiny->{error} = "protocol ( $HTTP_Tiny->{protocol} ) should be http or https";
-  undef
-  }
-
-$HTTP_Tiny
-}
-
-
-
 #   pass through the request to elasticsearch
 sub direct
 {
 my $obj = ((exists $_[0]) && (__PACKAGE__ eq ref $_[0])) ? shift : __NOT_AN_OBJECT([caller 0]->[3],@_);
 $obj->{error}     = 0;
 $obj->{scroll}    = undef;
-$obj->{send_data} = undef;
+$obj->{send_data} = '';
 $obj->{param}     = '';
 $obj->{sort}      = [];
 
@@ -298,7 +272,7 @@ $obj->{sort}      = [];
   }
 
   if ( $obj->{method} !~/(?i)^(GET|DELETE|PUT|POST)$/ ) {
-  $obj->{error} = "Unknown request method $arg{method}";
+  $obj->{error} = "Unknown request method $obj->{method}";
   return undef
   }
 
@@ -676,19 +650,20 @@ Search::ElasticDirect - An interface to ElasticSearch API, version independent
 
 =head1 VERSION
 
-version 2.2.0
+version 2.5.0
 
 =head1 SYNOPSIS
 
   use Search::ElasticDirect;
 
-  my $obj = Search::ElasticDirect->new(host=>'elk1.lan', port=>9200, protocol=>'http', timeout=>240, keep_alive=>'no', verify_SSL=>'no') or die;
-  my $elk = $obj->connect(protocol=>'http', authentication=> 'no', keep_alive=> 'yes');
+  my $elk = Search::ElasticDirect->new( host=> 'SomeServer',  protocol=> 'http', authentication=> 'no' );
+  die "Connection error because $elk->{error}\n" if $elk->{error};
 
   my $reply = $elk->direct( method=>'GET', path=>'_cat/nodes', param=>'h=name,ip,role,version&s=name:asc', format=>'data' );
   die "$elk->{error}\n" if $elk->{error};
 
   die "Allowed to run only at active master\n" unless $elk->ActiveMaster;
+  die $elk->{error} if $elk->{error};
 
   foreach my $srv (@{$reply}) {
   say join ',', $srv->{name}, $srv->{ip}, $srv->{version}, $srv->{role}
@@ -706,56 +681,106 @@ Check for errors the property  $obj->{error} . if everything is ok it is B<FALSE
 
 =head1 METHODS
 
-=head2 connect
+=head2 new
 
-For all the connection varieties, as first step, we must create an object using the new(...) e.g
-
-  my $obj = Search::ElasticDirect->new(host=>'elk1', port=>9200, protocol=>'https', timeout=>240, keep_alive=>'no', verify_SSL=>'no') or die;
+Create a new ElasticDirect object and define the connection parameters to the elasticsearch cluster. 
 
 =head3 https, username, password, CA certificate
 
 Most of the times you will use this one
 
-  my $elk = $obj->connect(authentication=> 'yes', certca=> '/etc/elasticsearch/chain.pem', username=> 'Joe', password=> 'JoePass', timeout=>3600, keep_alive=>'yes', verify_SSL=>'no');
+  my $elk = Search::ElasticDirect->new(
+  
+    host           => 'SomeServer',
+    port           => 9200,
+    protocol       => 'https',
+    timeout        => 1800,
+    keep_alive     => 'yes',
+    authentication => 'yes',
+    username       => 'Joe',
+    password       => 'JoePass',
+    certca         => '/etc/elasticsearch/chain.pem',
+    verify_SSL     => 'no' );
+  
   die $elk->{error} if $elk->{error};
 
 =head3 https, CA certificate, server key, server certificate
 
 If you are using the B<SearchGuard> security plugin and you need node admin key/cert without username and password to perform admin tasks e.g. to delete an immutable index
 
-  my $elk = $obj->connect(authentication=>'no', certca=> 'chain.pem', cert=> 'admin.pem', key=> 'admin.key', timeout=>3600, keep_alive=>'yes', verify_SSL=>'no');
+  my $elk = Search::ElasticDirect->new(
+  
+    host           => 'SomeServer',
+    port           => 9200,
+    protocol       => 'https',
+    timeout        => 1800,
+    keep_alive     => 'yes',
+    authentication => 'no',
+    certca         => '/etc/elasticsearch/chain.pem',
+    cert           => '/etc/elasticsearch/SomeServer.pem',
+    key            => '/etc/elasticsearch/SomeServer.key',  
+    verify_SSL     =>'no' );
+  
   die $elk->{error} if $elk->{error};
 
 =head3 https, username, password
 
 Without certificate
 
-  my $elk = $obj->connect(authentication=> 'yes', username=> 'Joe', password=> 'JoePass', timeout=>3600, keep_alive=>'yes');
+  my $elk = Search::ElasticDirect->new(
+  
+    host           => 'SomeServer',
+    port           => 9200,
+    protocol       => 'https',
+    timeout        => 1800,
+    keep_alive     => 'yes',
+    authentication => 'yes',
+    username       => 'Joe',
+    password       => 'JoePass' );
+  
   die $elk->{error} if $elk->{error};
 
 =head3 http, username, password
 
 Plain text with authentication
 
-  my $elk = $obj->connect(protocol=>'http', authentication=> 'yes', username=> 'Joe', password=> 'SomePass', keep_alive=> 'yes');
+  my $elk = Search::ElasticDirect->new(
+
+    host           => 'SomeServer',
+    port           => 9200,
+    protocol       => 'http',
+    timeout        => 1800,
+    keep_alive     => 'yes',
+    authentication => 'yes',
+    username       => 'Joe',
+    password       => 'SomePass' );
+
   die $elk->{error} if $elk->{error};
 
 =head3 http
 
 Plain text without authentication
 
-  my $elk = $obj->connect(protocol=>'http', authentication=> 'no', keep_alive=> 'yes');
+  my $elk = Search::ElasticDirect->new(
+
+    host           => 'SomeServer',
+    port           => 9200,
+    protocol       => 'http',
+    timeout        => 1800,
+    keep_alive     => 'yes',
+    authentication => 'no' );
+
   die $elk->{error} if $elk->{error};
 
 =head2 ActiveMaster
 
 If you have schedule your script at multiple servers for high availability but it must run once at active master server
 
-  my  $obj = Search::ElasticDirect->new(...) or die;
-  my  $elk = $obj->connect(...);
+  my  $obj = Search::ElasticDirect->new(...);
   die $elk->{error} if $elk->{error};
 
-  die "This server is not the active master\n" unless $elk->ActiveMaster;
+  die "Allowed to run only at active master\n" unless $elk->ActiveMaster;
+  die $elk->{error} if $elk->{error};
 
 =head2 direct
 
